import { injectable, inject, named, tagged } from 'inversify';
import * as Logger from 'bunyan';
import { ServiceBroker } from 'moleculer';
import { TaggedType } from '@common-stack/core';
import { IOutputService, IPodsRes, IOutputSettings, INewOutputPod, IOutputRes } from '../interfaces';
import { OutputCommands, TYPES, NATS_HEMERA_KUBE_OUTPUT } from '../constants';

@injectable()
export class OutputMicroservice implements IOutputService {
    constructor(
        @inject('MoleculerBroker')
        private broker: ServiceBroker,
        @inject(TYPES.IOutputSettings)
        @named(TaggedType.MICROSERVICE) private settings: IOutputSettings,
    ) {}
    private topic = NATS_HEMERA_KUBE_OUTPUT;

    public addNewType(podName: string, newItem: INewOutputPod): Promise<void> {
        return this.callAction<void>(OutputCommands.ADD_OUTPUT_POD, {podName, newItem});
    }

    public getPods(): Promise<IPodsRes[]> {
        return this.callAction<IPodsRes[]>(OutputCommands.GET_OUTPUT_PODS);
    }

    public getOutput(request: string) {
        return this.callAction<IOutputRes>(OutputCommands.GET_OUTPUT_DATA, { request });
    }

    public unWatchLogs() {
        return this.callAction<Boolean>(OutputCommands.UNWATCH_OUTPUT);
    }

    public dispose() {
        return this.callAction<void>(OutputCommands.DISPOSE_OUTPUT);
    }
    private async callAction<T>(command: string, params?: any) {
        return await this.broker.call<T, any>(`${this.topic}.${command}`, params);
    }
}
